require 'ravengar/all'
using Ravengar

describe 'base library' do
  let(:source) { %w(This is SOMEthing aWesoME) }
  describe 'filter' do
    it 'is called by the method name' do
      result = !(source | filter(->(it) { it =~ /^[A-Z]/ }) )
      expect(result).to eq(%w(This SOMEthing))
    end
    it 'is called using "~"' do
      result = !(source | ~->(it) { it =~ /^[A-Z]/ } )
      expect(result).to eq(%w(This SOMEthing))
    end
    it 'reverses the filter with "!"' do
      result = !(source | !->(it) { it =~ /^[A-Z]/ } )
      expect(result).to eq(%w(is aWesoME))
    end
  end
  describe 'show' do
    it 'outputs data to arg' do
      result = ""
      !(source | show(result))
      expect(result).to eq %("This"\n"is"\n"SOMEthing"\n"aWesoME"\n)
    end
    it 'outputs data to STDOUT with no args' do
      begin
        result = StringIO.new
        $stdout, previous_stdout = result, $stdout
        !(source | show)
        expect(result.string).to eq %("This"\n"is"\n"SOMEthing"\n"aWesoME"\n)
      ensure
        $stdout = previous_stdout
      end
    end
    it 'outputs data using other separator' do
      result = ""
      !(source | show(result, separator: '|'))
      expect(result).to eq %("This"|"is"|"SOMEthing"|"aWesoME"|)
    end
  end
  describe 'replicator' do
    it 'outputs the same thing to n lambdas' do
      result = []
      collector = ->(it) { result << it }
      !(source | replicator(collector, collector))
      expect(result).to eq %w(This This is is SOMEthing SOMEthing aWesoME aWesoME)
    end
  end
  describe 'diffuser' do
    it 'sends each output to a different lambda' do
      result1 = []
      result2 = []
      collector1 = ->(it) { result1 << it }
      collector2 = ->(it) { result2 << it }
      !(source | diffuser(collector1, collector2))
      expect(result1).to eq %w(This SOMEthing)
      expect(result2).to eq %w(is aWesoME)
    end
  end
end
